library IEEE;
use IEEE.std_logic_1164.all;            -- defines std_logic types

entity I60LOOP is
  port
    (
      -- PIC interface signals --
      RW   : in    std_logic;
      ICLK : in    std_logic;
--      BUSY : out   std_logic;
--      ATT  : out   std_logic;
      DATA : inout std_logic_vector (7 downto 0);  -- bus interface signals --

      Synclk : in std_logic;
      PLLclk : in std_logic;

      IOBits : inout std_logic_vector (95 downto 0);

      -- led bits
      LEDS : out std_logic_vector(7 downto 0)

      );
end I60LOOP;

architecture dataflow of I60LOOP is


-- misc global signals --
  signal DBUS   : std_logic_vector (7 downto 0);
  signal Alatch : std_logic_vector (7 downto 0);

  signal ICLKRise : std_logic;
  signal ICLKFall : std_logic;
  signal ICLKDel1 : std_logic;
  signal ICLKDel2 : std_logic;

-- decodes --   
  signal IOselect  : std_logic;
  signal DDRRead   : std_logic;
  signal PortRead  : std_logic;
  signal DDRWrite  : std_logic;
  signal PortWrite : std_logic;

  signal LoadPortCmd : std_logic_vector(11 downto 0);
  signal ReadPortCmd : std_logic_vector(11 downto 0);

  signal LoadDDRCmd : std_logic_vector(11 downto 0);
  signal ReadDDRCmd : std_logic_vector(11 downto 0);

  function OneOfTwelveDecode(ena : std_logic; dec : std_logic_vector(3 downto 0)) return std_logic_vector is
    variable result : std_logic_vector(11 downto 0);
  begin
    if ena = '1' then
      case dec is
        when "0000" => result := "000000000001";
        when "0001" => result := "000000000010";
        when "0010" => result := "000000000100";
        when "0011" => result := "000000001000";
        when "0100" => result := "000000010000";
        when "0101" => result := "000000100000";
        when "0110" => result := "000001000000";
        when "0111" => result := "000010000000";
        when "1000" => result := "000100000000";
        when "1001" => result := "001000000000";
        when "1010" => result := "010000000000";
        when "1011" => result := "100000000000";
        when others => result := "000000000000";
      end case;
    else
      result := "000000000000";
    end if;
    return result;
  end OneOfTwelveDecode;

  -- components --      
  component BytePort is
    port (
      clear    : in  std_logic;
      clk      : in  std_logic;
      ibus     : in  std_logic_vector (7 downto 0);
      obus     : out std_logic_vector (7 downto 0);
      loadport : in  std_logic;
      loadddr  : in  std_logic;
      readddr  : in  std_logic;
      portdata : out std_logic_vector (7 downto 0)
      );
  end component BytePort;

  component ByteIORB is
    port (
      obus     : out std_logic_vector (7 downto 0);
      readport : in  std_logic;
      portdata : in  std_logic_vector (7 downto 0));
  end component ByteIORB;

  component ledblink is
    port (
      clk  : in  std_logic;
      ledx : out std_logic_vector (7 downto 0)
      );
  end component ledblink;

  
begin

  gledblink : ledblink port map (
    clk  => PllClk,
    ledx => LEDS
    );  

  makeoports : for i in 0 to 11 generate
    oportx : BytePort port map (
      clear    => '0',
      clk      => SynClk,
      ibus     => Data,
      obus     => DBus,
      loadport => LoadPortCmd(i),
      loadddr  => LoadDDRCmd(i),
      readddr  => ReadDDRCmd(i),
      portdata => IOBits((((i+1)*8) -1) downto (i*8))
      );        
  end generate;

  makeiports : for i in 0 to 11 generate
    iportx : ByteIORB port map (
      obus     => DBus,
      readport => ReadPortCmd(i),
      portdata => IOBits((((i+1)*8) -1) downto (i*8))
      );        
  end generate;


--  The PIC bus interface

  ICLKEdgeDetect : process (synclk)
  begin
    if synclk'event and synclk = '1' then
      ICLKDel1 <= ICLK;
      ICLKDel2 <= ICLKDel1;
      if ICLKDel1 = '0' and ICLKDel2 = '1' then
        ICLKFall <= '1';
      else
        ICLKFall <= '0';
      end if;

      if ICLKDel1 = '1' and ICLKDel2 = '0' then
        ICLKRise <= '1';
      else
        ICLKRise <= '0';
      end if;
    end if;  -- clk
  end process;

  ALatcpproc : process (synclk)  -- address latch
  begin
    if synclk'event and synclk = '1' then
      if ICLKRise = '1' and RW = '1' then
        Alatch <= DATA;
      end if;
    end if;
  end process;

  DATADrivers : process (DBus, RW)
  begin
    if RW = '0' then
      DATA <= DBus;
    else
      DATA <= "ZZZZZZZZ";
    end if;
  end process DATADrivers;

  Decode : process (Alatch, DDRRead, DDRWrite, ICLKFall, IOSelect, PortRead,
                    PortWrite, RW)

  begin
    if Alatch(7 downto 5) = "110" then  -- base address of C0
      IOSelect <= '1';
    else
      IOSelect <= '0';
    end if;

    if Alatch(0) = '0' and IOSelect = '1' and RW = '1' and ICLKFall = '1' then
      PortWrite <= '1';
    else
      PortWrite <= '0';
    end if;
    if Alatch(0) = '1' and IOSelect = '1' and RW = '1' and ICLKFall = '1' then
      DDRWrite <= '1';
    else
      DDRWrite <= '0';
    end if;

    if Alatch(0) = '0' and IOSelect = '1' and RW = '0' then
      PortRead <= '1';
    else
      PortRead <= '0';
    end if;
    if Alatch(0) = '1' and IOSelect = '1' and RW = '0' then
      DDRRead <= '1';
    else
      DDRRead <= '0';
    end if;

    LoadPortCmd <= OneOfTwelveDecode(PortWrite, Alatch(4 downto 1));
    ReadPortCmd <= OneOfTwelveDecode(PortRead, Alatch(4 downto 1));
    LoadDDRCmd  <= OneOfTwelveDecode(DDRWrite, Alatch(4 downto 1));
    ReadDDRCmd  <= OneOfTwelveDecode(DDRRead, Alatch(4 downto 1));
    
  end process decode;


end Dataflow;
