library IEEE;
use IEEE.std_logic_1164.all;            -- defines std_logic types

entity Wavegen is
  port
    (
      -- PIC interface signals --
      RW   : in    std_logic;
      ICLK : in    std_logic;
--      BUSY : out   std_logic;
--      ATT  : out   std_logic;
      DATA : inout std_logic_vector (7 downto 0);  -- bus interface signals --

      Synclk : in std_logic;
      PLLclk : in std_logic;

      IOBits : inout std_logic_vector (95 downto 0);

      -- led bits
      LEDS : out std_logic_vector(7 downto 0)

      );
end WaveGen;

architecture dataflow of Wavegen is


-- misc global signals --
  signal DBUS   : std_logic_vector (7 downto 0);
  signal Alatch : std_logic_vector (7 downto 0);

  signal ICLKRise : std_logic;
  signal ICLKFall : std_logic;
  signal ICLKDel1 : std_logic;
  signal ICLKDel2 : std_logic;

-- decodes --   
  signal IOselect  : std_logic;
  signal FreqWrite  : std_logic;
  signal PtrWrite  : std_logic;
  signal DataWrite : std_logic;

  signal LoadFreqCmd : std_logic_vector(11 downto 0);
  signal LoadPtrCmd : std_logic_vector(11 downto 0);
  signal LoadDataCmd : std_logic_vector(11 downto 0);
  signal Readsignature : std_logic;

  function OneOfTwelveDecode(ena : std_logic; dec : std_logic_vector(3 downto 0)) return std_logic_vector is
    variable result : std_logic_vector(11 downto 0);
  begin
    if ena = '1' then
      case dec is
        when "0000" => result := "000000000001";
        when "0001" => result := "000000000010";
        when "0010" => result := "000000000100";
        when "0011" => result := "000000001000";
        when "0100" => result := "000000010000";
        when "0101" => result := "000000100000";
        when "0110" => result := "000001000000";
        when "0111" => result := "000010000000";
        when "1000" => result := "000100000000";
        when "1001" => result := "001000000000";
        when "1010" => result := "010000000000";
        when "1011" => result := "100000000000";
        when others => result := "000000000000";
      end case;
    else
      result := "000000000000";
    end if;
    return result;
  end OneOfTwelveDecode;

  -- components --      
  component PhaseAcc is
    port (
      clk      : in  std_logic;
      ibus     : in  std_logic_vector (7 downto 0);
	  addr	   : in std_logic_vector (3 downto 0);
      loadfreq : in  std_logic;
      loadptr  : in  std_logic;
      loaddata  : in  std_logic;
      phaseout : out std_logic_vector (7 downto 0)
      );
  end component Phaseacc;

--  component ledblink is
--    port (
--      clk  : in  std_logic;
--      ledx : out std_logic_vector (7 downto 0)
--      );
--  end component ledblink;

  
begin

 -- gledblink : ledblink port map (
 --   clk  => synclk,
 --   ledx => LEDS
 --   );  

  makephase : for i in 0 to 11 generate
    oportx : PhaseAcc port map (
      clk      => pllclk,
      ibus     => Data, 
	  addr	   => Alatch(3 downto 0),		  
      loadfreq => LoadFreqCmd(i),
      loadptr  => loadPtrCmd(i),
      loaddata => loadDataCmd(i), 
      phaseout =>IOBits((((i+1)*8) -1) downto (i*8))
      );        
  end generate;


--  The PIC bus interface

  ICLKEdgeDetect : process (pllclk)
  begin
    if pllclk'event and pllclk = '1' then
      ICLKDel1 <= ICLK;
      ICLKDel2 <= ICLKDel1;
      if ICLKDel1 = '0' and ICLKDel2 = '1' then
        ICLKFall <= '1';
      else
        ICLKFall <= '0';
      end if;

      if ICLKDel1 = '1' and ICLKDel2 = '0' then
        ICLKRise <= '1';
      else
        ICLKRise <= '0';
      end if;
    end if;  -- clk
  end process;

  ALatcpproc : process (pllclk)  -- address latch
  begin
    if pllclk'event and pllclk = '1' then
      if ICLKRise = '1' and RW = '1' then
        Alatch <= DATA;
      end if;
    end if;
  end process;

  DATADrivers : process (DBus, RW)
  begin
    if RW = '0' then
      DATA <= DBus;
    else
      DATA <= "ZZZZZZZZ";
    end if;
  end process DATADrivers;

  Decode : process (Alatch, FreqWrite, PtrWrite, ICLKFall,
                    DataWrite, RW)

  begin
   
    if Alatch(3) = '0' and RW = '1' and ICLKFall = '1' then
      FreqWrite <= '1';
    else
      FreqWrite <= '0';
    end if;
    if Alatch(3 downto 1) = "100" and RW = '1' and ICLKFall = '1' then
      PtrWrite <= '1';
    else
      PtrWrite <= '0';
    end if;
    if Alatch(3 downto 1) = "101" and RW = '1' and ICLKFall = '1' then
      DataWrite <= '1';
    else
      DataWrite <= '0';
    end if;
	
	if alatch = x"E0" and RW = '0' then
		DBus <= x"A5";
	else
		DBus <= "ZZZZZZZZ";
	end if;

    LoadFreqCmd <= OneOfTwelveDecode(FreqWrite, Alatch(7 downto 4));
    LoadPtrCmd <= OneOfTwelveDecode(PtrWrite, Alatch(7 downto 4));
    LoadDataCmd  <= OneOfTwelveDecode(DataWrite, Alatch(7 downto 4));
    
  end process decode;

   DriveLEDS : process(IOBITS)
  begin
    LEDS <= not IOBITS(7 downto 0);
  end process;

end Dataflow;
